from frozendict.core import frozendict as FrozendictPyClass
from .common import FrozendictCommonTest
from .subclass_only import FrozendictSubclassOnlyTest
import frozendict as cool
from frozendict import frozendict as FrozendictClass


is_subclass = True


class FrozendictPySubclass(FrozendictPyClass):
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls, *args, **kwargs)


class FrozendictPyMissingSubclass(FrozendictPyClass):
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls, *args, **kwargs)
    
    def __missing__(self, key):
        return key


class TestFrozendictPySubclass(
        FrozendictCommonTest, 
        FrozendictSubclassOnlyTest
):
    FrozendictClass = FrozendictPySubclass
    FrozendictMissingClass = FrozendictPyMissingSubclass
    c_ext = False
    is_subclass = is_subclass


if cool.c_ext:
    class FrozendictSubclass(FrozendictClass):
        def __new__(cls, *args, **kwargs):
                return super().__new__(cls, *args, **kwargs)


    class FrozendictMissingSubclass(FrozendictClass):
        def __new__(cls, *args, **kwargs):
                return super().__new__(cls, *args, **kwargs)
        
        def __missing__(self, key):
            return key
    
    
    class TestFrozendictSubclass(
            FrozendictCommonTest, 
            FrozendictSubclassOnlyTest
    ):
        FrozendictClass = FrozendictSubclass
        FrozendictMissingClass = FrozendictMissingSubclass
        c_ext = cool.c_ext
        is_subclass = is_subclass



from frozendict import frozendict
import copy

d = dict(a=1, b=2)
fd = frozendict(a=1, b=2)
reveal_type(d["a"])
reveal_type(fd["a"])
reveal_type(iter(d))
reveal_type(iter(fd))
reveal_type(dict.fromkeys("abc", 0))
reveal_type(frozendict.fromkeys("abc", 0))
reveal_type(d.copy())
reveal_type(fd.copy())
reveal_type(tuple(d.items())[0])
reveal_type(fd.item(0))
reveal_type(tuple(d.keys())[0])
reveal_type(fd.key(0))
reveal_type(tuple(d.values())[0])
reveal_type(fd.value(0))
reveal_type(d.get("a"))
reveal_type(fd.get("a"))
reveal_type(d.items())
reveal_type(fd.items())
reveal_type(d.keys())
reveal_type(fd.keys())
reveal_type(d.values())
reveal_type(fd.values())
reveal_type(copy.copy(d))
reveal_type(copy.copy(fd))
reveal_type(copy.deepcopy(d))
reveal_type(copy.deepcopy(fd))

reveal_type(reversed(fd))
reveal_type(fd | {1: 2})
reveal_type(fd.setdefault("a", 0))
reveal_type(fd.set("abc", 1))
reveal_type(fd.delete("a"))



from __future__ import annotations

from typing import TypeVar, overload, Type, Optional, Generic

try:
    from typing import Mapping, Sequence, Iterable, Iterator, Tuple
except ImportError:
    from collections.abc import Mapping, Sequence, Iterable, Iterator
    Tuple = tuple

K = TypeVar("K")
V = TypeVar("V")
SelfT = TypeVar("SelfT", bound=frozendict[K, V])

class frozendict(Mapping[K, V]):
    @overload
    def __new__(cls: Type[SelfT], **kwargs: V) -> frozendict[str, V]: ...
    @overload
    def __new__(cls: Type[SelfT], mapping: Mapping[K, V]) -> SelfT: ...
    @overload
    def __new__(cls: Type[SelfT], iterable: Iterable[Tuple[K, V]]) -> SelfT: ...
    
    def __getitem__(self: SelfT, key: K) -> V: ...
    def __len__(self: SelfT) -> int: ...
    def __iter__(self: SelfT) -> Iterator[K]: ...
    def __hash__(self: SelfT) -> int: ...
    def __reversed__(self: SelfT) -> Iterator[K]: ...
    def copy(self: SelfT) -> SelfT: ...
    def __copy__(self: SelfT) -> SelfT: ...
    def __deepcopy__(self: SelfT) -> SelfT: ...
    def set(self: SelfT, key: K, value: V) -> SelfT: ...
    def setdefault(self: SelfT, key: K, default: V) -> SelfT: ...
    def delete(self: SelfT, key: K) -> SelfT: ...
    def key(self: SelfT, index: int) -> K: ...
    def value(self: SelfT, index: int) -> V: ...
    def item(self: SelfT, index: int) -> Tuple[K, V]: ...
    def __or__(self: SelfT, other: Mapping[K, V]) -> SelfT: ...
    
    @classmethod
    def fromkeys(
        cls: Type[SelfT], 
        seq: Iterable[K], 
        value: Optional[V] = None
    ) -> SelfT: ...


FrozenOrderedDict = frozendict
